% clustering_genetree_BHV()
% 
%  Main procedure for gene clustering
%    
%   This matlab code is used for clustering gene trees based on geodesic
%   distance matrixin BHV tree space. It is used for 
%      "Multi-locus Phylogenetic Analysis with Gene Tree Clustering"
%      by Ruriko Yoshida, Kenji Fukumizu, and Chrysafis Vogiatzis
%
%    INPUT: the data files are given in the form of distance matrices
%    OUTPUT: clustering accuracy and graph%s
%
%   Required toolboxes by others
%     drtoolbox:  by van der Maaten
%             https://lvdmaaten.github.io/drtoolbox/
%     Ncut_9:  by Timothee Cour, Stella Yu, Jianbo Shi
%             http://www.timotheecour.com/software/ncut/ncut.html
%
%   Date: Feb 24, 2016
%   Version: 1.0
%   Author: Kenji Fukumizu
%   Affiliation:  The Institute of Statistical Mathematics, ROIS
%   (c) Kenji Fukumizu
%-------------------------------------------------------
%
%   function clustering_genetree_BHV(depth,clust,method,drmethod,dim,nbCluster)
%
%   depth: c for the ration of depth; depth = c*Ne
%   clust: clustering method "ncut", "kmean", "hclust"
%   method: reconstruction method for gene trees "NJp", "GTR", "HKY",
%           "K80", and $JC"
%   drmethod: methods for dimension reduction "kpca", "tsne"
%   dim: dimensionality of dimension reduction methods (3 or 2)
%   nbCluster: the number of clusters (currently, two clusters is supported for evaluation)
%
%-----------------------------------------------------------------------

function cls=clustering_genetree_BHV(depth,clust,method,drmethod,dim,nbCluster)

close all;

DATA_DIR='geodesic_new2';     % directory for BHV geodesic data

DISP=1; % 1: display the dimension reduction result 
SAVEIMG=1;  % 1: save the image
SAVE_DIR='ResultsImg';

SAVE=1;
CLS_DIR='ResultsClst';

dep=num2str(depth);

switch method
    case 'NJp'
        mname=method;
        disp_name=method;
    case {'GTR', 'HKY', 'K80', 'JC'};
        mname=sprintf('PHYML_%s',method);
        disp_name=sprintf('PHYML-%s',method);
    otherwise
        error('method must be either of NJp, GTR, HKY, K80, JC');
end

fname=sprintf('%s/geodesics%smix_seq_%s.txt',DATA_DIR,dep,mname);


% Geodesic distance data
G=dlmread(fname);
[n,m]=size(G);

% dimension reduction (kpca/t-sne)
switch drmethod
    case 'kpca'
        % kernel PCA
        sigma=1.2*median(reshape(G,n*n,1));
        A=exp(-G.*G./(2*sigma*sigma)); 
        H=eye(n)-ones(n,n)./n;
        K=H*A*H;
        K=(K+K')./2;    % for numerical reason
        [V D]=eigs(K,dim);
        U=V*sqrt(D);
    
    case 'tsne'        
        % t-SNE (requires drtoolbox by van der Maaten)
        U = tsne_d(G,[], dim, 30);  % function in drtoolbox 
            % perplexity 30 is a default value 
         % similarity matrix 
         
    %case 'isomap'
        %no_dims=dim;
        %k=12;       % k in kNN to make a neighborhood graph
        %[U, mapping] = isomap_G(G, no_dims, k);
        %ss=1.2*MedianDist(U);
        %ab=U*U';
        %aa=diag(ab);
        %D2=2*ab - repmat(aa,1,n) - repmat(aa',n,1);
        %W=exp(D2./(2*ss*ss));           % similarity matrix 
    case 'direct'
        ss=1.2*median(reshape(G,n*n,1));
        W=exp(-G.*G./(2*ss*ss));
    otherwise
        error('drmethod: kpca or tsne');
end

cls=zeros(n,nbCluster);
switch clust
    case 'Ncut'
        % Ncut clustering (requires Ncut9)
        if ~strcmp(drmethod,'direct')
            ss=1.2*MedianDist(U);
            ab=U*U';
            aa=diag(ab);
            D2=2*ab - repmat(aa,1,n) - repmat(aa',n,1);
            W=exp(D2./(2*ss*ss));           % similarity matrix with Gaussian kernel
        end
        [cls,NcutEigenvectors,NcutEigenvalues] = ncutW(W,nbCluster);
    case 'Kmean'
        KMidx=kmeans(U,nbCluster);
        for j=1:nbCluster
            idx= (KMidx==j);
            cls(idx,j)=1;
        end
    case 'Hclust'
        if strcmp(drmethod,'direct')
            D=G;
        else
            ab=U*U';
            aa=diag(ab);
            D=sqrt(max(-2*ab + repmat(aa,1,n) + repmat(aa',n,1),0));  % distance matrix
        end
        Z=linkage(D,'average');     % average linkage is used
        HCidx=cluster(Z,'maxclust',nbCluster);
        for j=1:nbCluster
            idx= (HCidx==j);
            cls(idx,j)=1;
        end
end


if DISP && ~strcmp(drmethod,'direct')
    cluster_color = ['rbgmyc'];
    figure;clf;
    
    if dim==2
        min_ax=min([min(U(:,1)),min(U(:,2))]);
        max_ax=max([max(U(:,1)),max(U(:,2))]);
        plot(U(:,1),U(:,2),'ko'); 
        axis(1.2*[min_ax max_ax min_ax max_ax]);
    elseif dim==3
        min_ax=min([min(U(:,1)),min(U(:,2)),min(U(:,3))]);
        max_ax=max([max(U(:,1)),max(U(:,2)),max(U(:,3))]);
        plot3(U(:,1),U(:,2),U(:,3),'ko'); 
        axis(1.2*[min_ax max_ax min_ax max_ax min_ax max_ax]);
    end
    axis square;
    hold on;

    for j=1:nbCluster
        id = find(cls(:,j));
        if dim==2
            plot(U(id,1),U(id,2),'ko', 'MarkerFaceColor',cluster_color(j)); 
        elseif dim==3
            plot3(U(id,1),U(id,2),U(id,3),'ko', 'MarkerFaceColor',cluster_color(j)); 
        end
    end
    stitle=sprintf('%s Clustering, %s, c=%.1f (%s)',clust,drmethod,depth,disp_name);
    title(stitle,'FontName','Times','FontSize',18);
    hold off; 
    drawnow;
end        


if DISP && SAVEIMG
    imgfile=sprintf('%s/Geod%.1f_%s_clustering_%s_%s_D%d_cls%d.new.eps',...
        SAVE_DIR,depth,clust,drmethod,mname,dim,nbCluster);
    saveas(gcf, imgfile,'psc2');
end

if SAVE
    clsfile=sprintf('%s/Geod%.1f_%s_%s_%s_D%d_cls%d.txt',...
        CLS_DIR,depth,clust,drmethod,mname,dim,nbCluster);
    dlmwrite(clsfile, cls, ' ');
end






